
"""
This module finds and classifies paths connecting nodes in a given network.
"""

def find_paths(network,src,tgt):
	"""
	Find all the paths connecting the src node to the tgt node.  The set of paths is returned as a
	list of paths.  Each path is a list of edges.
	"""
	plist = []
	
	path = []
	node_stack = [src]
	iterators = [network.E(src).__iter__()]
	while len(node_stack) > 0:
		
		node = node_stack[-1]
		iterator = iterators[-1]
		
		try:
			edge = iterator.next()
			dest = network.target(edge)
		
			if dest == tgt:
				path.append(edge)
				plist.append(list(path))
				path.pop()
			elif dest not in node_stack:
				path.append(edge)
				node_stack.append(dest)
				iterators.append(network.E(dest).__iter__())
		except StopIteration:
			node_stack.pop()
			iterators.pop()
			
			if len(path) > 0:
				path.pop()
	
	return plist

def classify_path(path):
	"""
	This method classifies a path as either positive ('+') or negative ('-')
	"""
	inhibs = filter(lambda x: x.head == '|', path)
	
	if len(inhibs) % 2 == 0:
		return '+'
	else:
		return '-'