import Tkinter as tk

from pathwayoracle.tkui.marking import MarkingManagerWindow, MarkingSeriesInspector, MarkingGroupInspector
import pathwayoracle.io.marking as marking

class MarkingManagerController:

	def __init__(self,mc):
		self.mc = mc
		
		# create the MarkingManager window
		self.mmw = MarkingManagerWindow(self)
		self.set_visible(False)
		
	def set_visible(self,visible):
		
		if visible:
			self.mmw.deiconify()
		else:
			self.mmw.withdraw()
	
	def get_visible(self):
		return self.mmw.state() != 'withdrawn'
		
	def inspect_series(self,mseries):
		MarkingSeriesInspectorController(mseries)
		
	def inspect_group(self,mgroup):
		MarkingGroupInspectorController(self,mgroup)
		
	def add_marking_group(self,mgroup):
		self.mmw.add_marking_group(mgroup)
	
	def add_marking_series(self,mseries):
		self.mmw.add_marking_series(mseries)
	
	def save_series(self,fname,mseries):
		marking.write_series(mseries,fname)
		
	def save_group(self,fname,mgroup):
		marking.write_group(mgroup,fname)

	def color_network(self,molecule_names,molecule_colors):
		"""
		Color the network using the molcule names and colors specified.
		"""
		self.mc.color_network(molecule_names,molecule_colors)
		
	def uncolor_network(self):
		"""
		Remove the colors that were applied to the network by any previous calls to color_network(...)
		"""
		self.mc.uncolor_network()
		
class MarkingSeriesInspectorController:
	
	def __init__(self,mseries):
		
		self.mseries = mseries
		self.ui = MarkingSeriesInspector(self,mseries)
		
class MarkingGroupInspectorController:
	
	def __init__(self,mmc,mgroup):
	
		self.mmc = mmc
		self.mgroup = mgroup
		self.ui = MarkingGroupInspector(self,mgroup)
		
	def color_network(self,molecule_names,molecule_colors):
		"""
		Color the network using the molcule names and colors specified.
		"""
		self.mmc.color_network(molecule_names,molecule_colors)
		
	def uncolor_network(self):
		"""
		Remove the colors that were applied to the network by any previous calls to color_network(...)
		"""
		self.mmc.uncolor_network()