import csv
from pathwayoracle.model.marking import *

def read_group(fname): 
	"""
	Load a marking group from a file.
	"""
	reader = csv.reader(open(fname,'r'))
	
	# read the header
	header = reader.next()
	header = header[1:]
	
	molecule2idx = {}
	for mname in header:
		molecule2idx[mname] = len(molecule2idx)
		
	# read the groups
	group2values = {}
	
	for line in reader:
		group2values[line[0]] = [float(x) for x in line[1:]]

	return MarkingGroup(group2values,molecule2idx,name=fname)
	

def write_group(mgroup,fname):
	"""
	Write marking group data to a file.
	"""
	fid = open(fname,'w')
	
	# write the header
	for mname in mgroup.molecule_names():
		fid.write(',' + mname)

	# write the group values
	for grp in mgroup.group_names():
		fid.write(grp)
		
		for mname in mgroup.molecule_names():
			fid.write(',' + str(mgroup.get_molecule_value(mname,grp)))
		
		fid.write('\n')

	fid.close()


def read_series(fname):
	"""
	Load a marking series from a file.
	"""
	reader = csv.reader(open(fname,'r'))
	
	num_markings = 0
	molecule2series = {}
	
	first = True
	for line in reader:
	
		# skip the header
		if first:
			first = False
			continue
	
		molecule2series[line[0]] = [float(x) for x in line[1:]]
		num_markings = len(line) - 1
	
	return MarkingSeries(molecule2series,num_markings,name=fname)


def write_series(mseries,fname):
	"""
	Write marking series data to a file.
	"""
	fid = open(fname,'w')
	
	# write a header
	fid.write('# Marking Series - do not remove this line\n')
	
	# write the molecule values
	for mname in mseries.molecule_names():
		fid.write(mname)
		
		for i in range(0,mseries.num_markings()):
			fid.write(',' + str(mseries.get_molecule_value(mname,i)))
			
		fid.write('\n')

	fid.close()