
"""
This module reads and writes paths files.  In all cases, the set of paths is a list
of paths where each path is a list of edges in a signaling network.
"""

def read(fname,network):
	"""
	Read a path set from a file.  network is the network to which the loaded paths should
	belong.
	"""
	paths = []
	fid = open(fname,'r')
	
	while fid:
		line = fid.readline()
		
		if len(line) == 0:
			break
			
		elements = line.split()
		
		path = []
		
		for i in range(0,len(elements)-2,2):
			src_name = elements[i]
			effect = elements[i+1][1]
			tgt_name = elements[i+2]
						
			# find the source and target
			src = None
			tgt = None
			
			for n in network.V:
				if n.label == src_name:
					src = n
				elif n.label == tgt_name:
					tgt = n
					
				if src and tgt:
					break
			
			if src is None:
				raise Exception, "Unable to find node " + src_name
				
			if tgt is None:
				raise Exception, "Unable to find node " + tgt_name
				
			# find the edge
			for e in network.edges(src):
				if e.head == effect and network.target(e) == tgt:
					path.append(e)
					break
					
		paths.append(path)
		
	return paths

def write(paths,network,fname):
	"""
	Write a path set to a file.  network is the network that the paths belong to.
	"""
	fid = open(fname,'w')
	
	for path in paths:
		fid.write(path2str(path,network) + '\n')
	
	fid.close()
	
def path2str(path,network):
	"""
	Convert a path to a string
	"""
	str_list = []
	for e in path:
		str_list.append(network.source(e).label)
		str_list.append('-' + e.head)
	str_list.append(network.target(path[-1]).label)

	return ' '.join(str_list)