
import threading
import re

import Tkinter as tk
import tkFileDialog
import tkMessageBox

import pyweaver.constants as pgc
import pyweaver.visual.tk as tkg

DEFAULT_NODE_COLOR = 'blue'

def parse_hi_list(hi_str):
	"""
	Parse the hi list format used to specify what the hi value is for a given protein.
	"""
	his = hi_str.split(',');
	print his
	hi_list = []
	p = re.compile(r'([^\s\(]+)\s*\(\s*(\d+)\s*\)')
	for hi in his:
		hi = hi.strip()
		m = re.match(p, hi)
		
		if m is None:
			raise Exception, 'Unable to parse high entry: %s' % hi
		
		hi_list.append((m.group(1),int(m.group(2))))
	
	return hi_list


class MainWindow:

	def __init__(self,controller,root,network):
		self.tk_root = root
		self.mc = controller
		self.network = network
		
		self.build_ui()
	
	def build_ui(self):
		self.tk_root.title("PathwayOracle")
		
		# create the canvas
		#self.gframe = tkg.GraphCanvas(self.tk_root,self.network)
		self.gframe = tkg.GraphFrame(self.tk_root,self.network)
		self.gframe.pack(fill="both",expand=True)
		
		# create the menu
		self.menu = tk.Menu(self.tk_root)
		self.tk_root.config(menu=self.menu)
		
		self.network_menu = tk.Menu(self.menu)
		self.menu.add_cascade(label="Network", menu=self.network_menu)
		self.network_menu.add_command(label="Save network...", command=lambda : self.save_network())
		
		self.data_menu = tk.Menu(self.menu)
		self.menu.add_cascade(label="Data", menu=self.data_menu)
		self.data_menu.add_command(label="Open marking series...", command=lambda : self.open_marking_series())
		self.data_menu.add_command(label="Open marking group...", command=lambda : self.open_marking_group())
		self.data_menu.add_command(label="Open path set...", command=lambda : self.open_path_set())
		
		self.struct_menu = tk.Menu(self.menu)
		self.menu.add_cascade(label="Structural Analysis", menu=self.struct_menu)
		self.struct_menu.add_command(label="Compute paths...", command=lambda : self.show_compute_paths_prompt())
		
		self.dyn_menu = tk.Menu(self.menu)
		self.menu.add_cascade(label="Dynamic Analysis", menu=self.dyn_menu)
		self.dyn_menu.add_command(label="New single simulation...", command=lambda : self.show_tok_sim_prompt())
		self.dyn_menu.add_command(label="New differential simulation...", command=lambda : self.show_diff_tok_sim_prompt())
	
		self.win_menu = tk.Menu(self.menu)
		self.menu.add_cascade(label="Window", menu=self.win_menu)
		self.win_menu.add_command(label="Marking manager", command=lambda : self.mc.toggle_mm_visibility())
	
	def open_marking_group(self):
		fname = tkFileDialog.askopenfilename(parent=self.tk_root, title="Select the marking group to open")
	
		if fname is not None:
			self.mc.open_marking_group(fname)
	
	def open_marking_series(self):
		fname = tkFileDialog.askopenfilename(parent=self.tk_root, title="Select the marking series to open")
	
		if fname is not None:
			self.mc.open_marking_series(fname)
	
	def open_path_set(self):
		fname = tkFileDialog.askopenfilename(parent=self.tk_root, title="Select the path set to open")
	
		if fname is not None:
			self.mc.open_path_set(fname)
	
	def show_tok_sim_prompt(self):
		tsp = TokenizedSimulationPrompt(self.mc)
	
	def show_diff_tok_sim_prompt(self):
		tsp = DifferentialTokenizedSimulationPrompt(self.mc)
	
	def save_network(self):
		self.gframe.sync_graph(True)
		
		fname = tkFileDialog.asksaveasfilename()
		
		self.mc.save_network(fname)
		
		tkMessageBox.showinfo(title="Network saved", message="Network was successfully saved.")
	
	def show_compute_paths_prompt(self):
		ComputePathsPrompt(self.mc)
	
	def show(self):
		
		class ShowThread(threading.Thread):
	
			def __init__(self,root):
				threading.Thread.__init__(self)
				self.root = root
				
			def run(self):
				tk.mainloop()
				
		thread = ShowThread()
		thread.start()
		
	def color(self,c,color):
		self.gframe.color(c,color)
		
	def reset_node_colors(self):
		self.gframe.color(pgc.NODES,DEFAULT_NODE_COLOR)
		
	def set_width(self,e,w):
		self.gframe.width(e,w)

class TokenizedSimulationPrompt(tk.Toplevel):
	"""
	This class is a window that presents the user with the configurations that must be
	configured in order to run a tokenized simulation.  Once this is done, the user can press
	"Run simulation" and run their simulation from this window.
	"""
	
	def __init__(self,controller):
		tk.Toplevel.__init__(self)
		self.title("New simulation")
		self.mc = controller
		
		self.build_ui()
		
	def build_ui(self):
		
		tk.Label(self, text="Result Marking Name").grid(row = 0)
		tk.Label(self, text="# of Runs").grid(row = 1)
		tk.Label(self, text="# of Blocks").grid(row = 2)
		tk.Label(self, text="High Molecules").grid(row = 3)
		tk.Label(self, text="Low Molecules").grid(row = 4)		

		
		self.mname_entry = tk.Entry(self, text="SimX")
		self.mname_entry.grid(row = 0, column = 1)
		
		self.nruns_entry = tk.Entry(self,text="100")
		self.nruns_entry.grid(row = 1, column = 1)
		
		self.nblocks_entry = tk.Entry(self,text="30")
		self.nblocks_entry.grid(row = 2, column = 1)
		
		self.high_mols = tk.Entry(self)
		self.high_mols.grid(row = 3, column = 1)
		
		self.low_mols = tk.Entry(self)
		self.low_mols.grid(row = 4, column = 1)
		
		tk.Button(self, text="Run simulation", command=lambda: self.run_sim()).grid(row = 5)
		
	def run_sim(self):
		try:
			hi_list = parse_hi_list(self.high_mols.get())
			low_mols = []
			if len(self.low_mols.get()) > 0:
				low_mols = self.low_mols.get().split(',')
				
			self.mc.run_simulation(self.mname_entry.get(),
								   int(self.nruns_entry.get()),
								   int(self.nblocks_entry.get()),
								   0,
								   hi_list,
								   low_mols)
		except Exception, detail:
			tkMessageBox.showinfo(title="Simulation failed", message=detail)

class DifferentialTokenizedSimulationPrompt(tk.Toplevel):
	"""
	This class is a window that presents the user with the configurations that must be
	configured in order to run a tokenized simulation.  Once this is done, the user can press
	"Run simulation" and run their simulation from this window.
	"""
	
	def __init__(self,controller):
		tk.Toplevel.__init__(self)
		self.title("New differential simulation")
		
		self.mc = controller
		
		self.build_ui()
		
	def build_ui(self):
		
		tk.Label(self, text="Result Marking Name").grid(row = 0)
		tk.Label(self, text="# of Runs").grid(row = 1)
		tk.Label(self, text="# of Blocks").grid(row = 2)
		tk.Label(self, text="High Molecules Cond. 1").grid(row = 3)
		tk.Label(self, text="Low Molecules Cond. 1").grid(row = 4)		
		tk.Label(self, text="High Molecules Cond. 2").grid(row = 5)
		tk.Label(self, text="Low Molecules Cond. 2").grid(row = 6)
		
		self.mname_entry = tk.Entry(self, text="SimX")
		self.mname_entry.grid(row = 0, column = 1)
		
		self.nruns_entry = tk.Entry(self,text="100")
		self.nruns_entry.grid(row = 1, column = 1)
		
		self.nblocks_entry = tk.Entry(self,text="30")
		self.nblocks_entry.grid(row = 2, column = 1)
		
		self.high_mols1 = tk.Entry(self)
		self.high_mols1.grid(row = 3, column = 1)
		
		self.low_mols1 = tk.Entry(self)
		self.low_mols1.grid(row = 4, column = 1)
		
		self.high_mols2 = tk.Entry(self)
		self.high_mols2.grid(row = 5, column = 1)
		
		self.low_mols2 = tk.Entry(self)
		self.low_mols2.grid(row = 6, column = 1)
		
		tk.Button(self, text="Run simulation", command=lambda: self.run_sim()).grid(row = 7)
		
	def run_sim(self):
		try:
			# parse high molecule settings
			hi1_list = parse_hi_list(self.high_mols1.get())
			hi2_list = parse_hi_list(self.high_mols2.get())
		
			low_mols1 = []
			if len(self.low_mols1.get()) > 0:
				low_mols1 = self.low_mols1.get().split(',')
		
			low_mols2 = []
			if len(self.low_mols2.get()) > 0:
				low_mols2 = self.low_mols2.get().split(',')
		
			self.mc.run_diff_sim(self.mname_entry.get(),
								   int(self.nruns_entry.get()),
								   int(self.nblocks_entry.get()),
								   0,
								   hi1_list,
								   low_mols1,
								   hi2_list,
								   low_mols2)
		except Exception, detail:
			tkMessageBox.showinfo(title='Simulation failed', message=detail)

class ComputePathsPrompt(tk.Toplevel):
	"""
	This class provides a prompt for computing the paths connecting a source node and a target node.
	"""
	
	def __init__(self,controller):
		tk.Toplevel.__init__(self)
		
		self.mc = controller
		
		self.build_ui()
		
	def build_ui(self):
		
		tk.Label(self, text="Source Node").grid(row=0)
		tk.Label(self, text="Target Node").grid(row=1)

		self.src_name = tk.Entry(self)
		self.src_name.grid(row=0,column=1)
		
		self.tgt_name = tk.Entry(self)
		self.tgt_name.grid(row=1,column=1)
		
		tk.Button(self, text="Compute paths", command=lambda: self.compute_paths()).grid(row=2)
	
	def compute_paths(self):
		self.mc.compute_paths(self.src_name.get(),self.tgt_name.get())