import Tkinter as tk
import tempfile as tf
import pylab
from PIL import Image,ImageTk
import tkFileDialog
import tkMessageBox
import types

from pathwayoracle.tkui.util.heatmap import Heatmap

class MarkingManagerWindow(tk.Toplevel):

	def __init__(self,mmc):
		tk.Toplevel.__init__(self)
		self.title("Marking Manager")
		
		self.mmc = mmc

		self.build_ui()
		
		self.marking_lookup = {}
		
	def build_ui(self):
		# make the list box
		self.list = tk.Listbox(self)
		self.list.pack(fill=tk.BOTH, expand=1)
		
		# # make the scroll bar
		# scrollY = tk.Scrollbar(self, orient=tk.VERTICAL)
		# 
		# self.list['yscrollcommand'] = scrollY.set
		# scrollY['command'] = self.list.yview
		# 
		# scrollY.pack(side=tk.RIGHT, fill=tk.Y)

		# make the buttons
		self.button_frame = tk.Frame(self)
		self.button_frame.pack(side=tk.BOTTOM,fill=tk.X)
		
		self.inspect = tk.Button(self.button_frame,text="Inspect",command=lambda: self.inspect_selected_marking())
		self.inspect.grid(row=0,column=0)
		
		self.save = tk.Button(self.button_frame,text="Save",command=lambda: self.save_selected_marking())
		self.save.grid(row=0,column=1)
		
		# Bind the window closing event to hiding the window (we shouldn't ever destroy this window)
		self.protocol('WM_DELETE_WINDOW', lambda: self.withdraw())
		
	def add_marking_series(self,mseries):
		self.list.insert(tk.END,mseries.get_name())
		
		self.marking_lookup[mseries.get_name()] = mseries
	
	def add_marking_group(self,mgroup):
		self.list.insert(tk.END,mgroup.get_name())
		
		self.marking_lookup[mgroup.get_name()] = mgroup
		
	def inspect_marking(self,marking):
		
		if marking.marking_type() == 'series':
			self.mmc.inspect_series(marking)
		elif marking.marking_type() == 'group':
			self.mmc.inspect_group(marking)
		
	def inspect_selected_marking(self):
		selected = self.list.get(tk.ACTIVE)
		
		if len(selected) == 0:
			return
		
		marking = self.marking_lookup[selected]
		
		self.inspect_marking(marking)
	
	def save_selected_marking(self):
		selected = self.list.get(tk.ACTIVE)
		
		if len(selected) == 0:
			return
		
		marking = self.marking_lookup[selected]
		
		# select a file
		fname = tkFileDialog.asksaveasfilename()
		
		if fname is None:
			return
		
		# save the marking
		if marking.marking_type() == 'series':
			self.mmc.save_series(fname,marking)
		elif marking.marking_type() == 'group':
			self.mmc.save_group(fname,marking)
			
		tkMessageBox.showinfo(title='Save successful', message='Marking ' + marking.get_name() + ' was successfully saved.')
			

class MarkingSeriesInspector(tk.Toplevel):
	"""
	Window for inspecting marking series data.
	"""

	def __init__(self,controller,mseries):
		tk.Toplevel.__init__(self)
		self.title("Marking Series Inspector")
		
		self.controller = controller
		self.mseries = mseries
		
		self.build_ui()
		
	def build_ui(self):
		
		# make check box panel
		check_frame = tk.Frame(self)
		check_frame.pack(side=tk.LEFT,fill=tk.Y,expand=0)
		
		scrollY = tk.Scrollbar(check_frame, orient=tk.VERTICAL)
		self.listbox = tk.Listbox(check_frame,selectmode=tk.MULTIPLE)
		
		self.listbox['yscrollcommand'] = scrollY.set
		scrollY['command'] = self.listbox.yview
		
		scrollY.pack(side=tk.RIGHT, fill=tk.Y)
		self.listbox.pack(side=tk.TOP,fill=tk.BOTH, expand=1)
		
		# handle selection events to the list
		self.curr_selection = self.listbox.curselection()
		self.poll()
				
		# populate listbox
		nrow = 0
		self.mname2state = {}
		mnames = list(self.mseries.molecule_names())
		mnames.sort()
		for m in mnames:
			self.listbox.insert(tk.END,m)
							
		# make rendering canvas
		self.plot_canvas = tk.Canvas(self)
		self.plot_canvas.pack(side=tk.TOP,fill=tk.BOTH,expand=1)
		self.plot_tkimage = None
		self.curr_image = None
		
		# handle canvas resizing
		self.plot_canvas.bind('<Configure>', self.canvas_configured)
		self.plot_canvas.curr_width = 100
		self.plot_canvas.curr_height = 100
		
		self.plotted_molecules = {}
		
		# make a bunch of free styles for plotting lines with
		colors = ['k','r','b','g','m']
		lines = ['-','--','-.',':']
		
		self.free_styles = [l + c for l in lines for c in colors]
		self.free_styles.reverse()
		
		# generate the empty plot
		self.refresh_plot()
	
	def canvas_configured(self,e):
		# extract the new dimensions of the canvas
		self.plot_canvas.curr_width = e.width
		self.plot_canvas.curr_height = e.height
		
		# redraw the plot so it's the right size
		self.refresh_plot()
	
	def poll(self):
		"""
		This is used for tracking changes to the listbox.
		"""
		now = self.listbox.curselection()
		
		if now != self.curr_selection:
			self.refresh_plot()
			self.curr_selection = now
		self.after(250, self.poll)
	
	def refresh_plot(self):
		
		selected_list = self.listbox.curselection()
		
		list_changed = selected_list != self.curr_selection or self.curr_image is None
		
		if list_changed:
			selected_items = set()
		
			for i in selected_list:
				if type(selected_list) == types.StringType:
					i = int(i)
				selected_items.add(self.listbox.get(i))

			for m in self.mseries.molecule_names():
				selected = m in selected_items
				if not selected and m in self.plotted_molecules:
					self.free_styles.append(self.plotted_molecules[m])
					del self.plotted_molecules[m]
				elif selected and m not in self.plotted_molecules:
					if len(self.free_styles) == 0:
						print "Maximum number of molecules plotted."
						self.mname2state[m].set(0)
					else:
						self.plotted_molecules[m] = self.free_styles.pop()


		# for m in self.mseries.molecule_names():
		# 	if self.mname2state[m].get() == 0 and m in self.plotted_molecules:
		# 		self.free_styles.append(self.plotted_molecules[m])
		# 		del self.plotted_molecules[m]
		# 	elif self.mname2state[m].get() == 1 and m not in self.plotted_molecules:
		# 		if len(self.free_styles) == 0:
		# 			print "Maximum number of molecules plotted."
		# 			self.mname2state[m].set(0)
		# 		else:
		# 			self.plotted_molecules[m] = self.free_styles.pop()
		
		# if the content of the plot changed, replot the data
		image = None
		if list_changed:
			fig = pylab.figure()
		
			rendered_molecules = []
			for m in self.plotted_molecules:
				pylab.plot(self.mseries.get_molecule_series(m),self.plotted_molecules[m])
		
			pylab.xlabel('Time blocks')
			pylab.ylabel('# of tokens')
			
			if len(self.plotted_molecules) > 0:
				pylab.legend(self.plotted_molecules,'upper left')
		
			# Create a temporary file
			fid = tf.NamedTemporaryFile(suffix='.png')
			fname = fid.name
			fid.close()
		
			# save the matplotlib figure
			pylab.savefig(fname)
		
			# load the new figure
			image = Image.open(fname)
		else:
			image = self.curr_image
			
		self.curr_image = image
		width = self.plot_canvas.curr_width
		height = self.plot_canvas.curr_height
		scaled_image = image.resize((width,height))
		photo = ImageTk.PhotoImage(scaled_image)
		
		# discard the current image
		if self.plot_tkimage:
			self.plot_canvas.delete(self.plot_tkimage)
			
		# plot the new one!
		self.plot_tkimage = self.plot_canvas.create_image(0,0,anchor=tk.NW,image=photo)
		self.plot_raw_image = scaled_image
		self.plot_raw_photo = photo
		
class MarkingGroupInspector(tk.Toplevel):
	"""
	UI for inspecting marking group information.  The core UI component is a heatmap showing the 
	relative values (using shades of red for negative values, shades of green for positive values, 
	and black for zero values).  Rows are groups, columns are molecules.  The user can select
	a row by clicking on the group name and apply the molecule colors in that group to the main
	graph view.
	"""

	def __init__(self,controller,mgroup):
		tk.Toplevel.__init__(self)
		self.title("Marking Group Inspector")
		
		self.controller = controller
		self.mgroup = mgroup
		
		self.build_ui()
		
	def destroy(self):
		self.controller.uncolor_network()
		
		tk.Toplevel.destroy(self)
	
	def build_ui(self):
		
		data = [[self.mgroup.get_molecule_value(m,g) for m in self.mgroup.molecule_names()] for g in self.mgroup.group_names()]
		
		self.heatmap = Heatmap(self,data,self.mgroup.group_names(),self.mgroup.molecule_names())
		self.heatmap.pack(side=tk.TOP,fill=tk.BOTH,expand=1)
		
		self.heatmap.add_row_listener(self)
		
	# Row Listener methods used by the heatmap
	def row_selected(self,hm,ridx):
		self.controller.color_network(self.heatmap.column_names(),self.heatmap.row_colors(ridx,style='pygraphrgb'))
	
	def row_deselected(self,hm,ridx):
		self.controller.uncolor_network()