import Tkinter as tk
import tkFileDialog
import tkMessageBox

class PathSetInspector(tk.Toplevel):

	def __init__(self,controller,path_info,network):
		tk.Toplevel.__init__(self)
		
		self.controller = controller
		self.path_info = path_info
		self.network = network
		
		self._build_ui()
		
		# poll for selection
		self.current_selection = []
		self.poll()
		
	def poll(self):
		next_selection = self.path_list.curselection()
		if self.current_selection != next_selection:
			self.update_selection(next_selection)
		self.after(250,self.poll)
	
	def update_selection(self,next_selection):
		
		# detect deselected paths
		for idx in self.current_selection:
			if idx not in next_selection:
				self.controller.path_deselected(self.path_lookup[self.path_list.get(idx)][0])
		
		# detect selected paths
		for idx in next_selection:
			if idx not in self.current_selection:
				self.controller.path_selected(self.path_lookup[self.path_list.get(idx)][0])
		
		# update current selection
		self.current_selection = next_selection
	
	def _build_ui(self):
	
		# Make the path list box
		self.path_list = tk.Listbox(self,selectmode=tk.MULTIPLE)
		self.path_list.pack(side=tk.TOP, fill=tk.BOTH, expand=1)
	
		# populate the list box
		self.path_lookup = {}
		for path, classification in self.path_info:
			sr = self.path2str(path)
			self.path_list.insert(tk.END, sr)
			self.path_lookup[sr] = (path,classification)
	
		# make the buttons
		self.btn_frame = tk.Frame(self)
		self.btn_frame.pack(side=tk.BOTTOM, fill=tk.X, expand=1)
		
		tk.Button(self.btn_frame, text="Select positives", command=lambda: self.select_positives()).grid(row=0,column=0)
		tk.Button(self.btn_frame, text="Select negatives", command=lambda: self.select_negatives()).grid(row=0,column=1)
		tk.Button(self.btn_frame, text="Deselect all", command=lambda: self.deselect()).grid(row=0,column=2)
		tk.Button(self.btn_frame, text="Save paths", command=lambda: self.save_paths()).grid(row=0,column=3)
		
	def select_positives(self):
		
		for i in range(0,self.path_list.size()):
			name = self.path_list.get(i)
			
			path, classification = self.path_lookup[name]
			
			if classification == '+':
				self.path_list.select_set(i)
	
	def select_negatives(self):
		for i in range(0,self.path_list.size()):
			name = self.path_list.get(i)
			
			path, classification = self.path_lookup[name]
			
			if classification == '-':
				self.path_list.select_set(i)
		
	def deselect(self):
		self.path_list.select_clear(0,tk.END)
		
	def save_paths(self):
		fname = tkFileDialog.asksaveasfilename()
		
		if fname is None:
			return
			
		self.controller.save_paths(fname)
		
		tkMessageBox.showinfo(title="Save successful",message="Paths saved to file " + fname)
		
	def path2str(self,path):
	
		str_list = []
		for e in path:
			str_list.append(self.network.source(e).label)
			str_list.append('-' + e.head)
		str_list.append(self.network.target(path[-1]).label)
	
		return ' '.join(str_list)