"""
This module contains a variety of functions for generating random trees.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import types
import random as rnd
from pyweaver.tree import *

__all__ = ['random']

def random(leaf_labels,node_generator= lambda x: Node(x)):
	"""
	Generate a random binary tree with the specified leaf labels.  leaf_labels is a list of the
	labels the tree should have.  The tree returned will be rooted and indexed by the label field.
	"""
	
	# create the random tree through random coalescence
	clades = list(leaf_labels)
	
	while len(clades) != 1:
		c1 = clades.pop(rnd.randrange(0,len(clades)))
		c2 = clades.pop(rnd.randrange(0,len(clades)))
		
		clades.append((c1,c2))
	
	#####
	# create the tree
	root = node_generator(None)
	T = RTree(root)
	T.N.index('label')
	
	def make_node(parent,clade):
		if type(clade) == types.TupleType:
			n = node_generator(None)
			T.add_child(parent,n)
			
			x,y = clade
			make_node(n,x)
			make_node(n,y)
		else: # this is a leaf
			n = node_generator(clade)
			T.add_child(parent,n)
	
	root_clade = clades.pop()
	left,right = root_clade
	make_node(root,left)
	make_node(root,right)
	
	return T