"""
Unit tests for the undirected graph.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import unittest
import pyweaver.graph as graph

class DiGraphTests(unittest.TestCase):

	def test_endpoints(self):
		"""
		Test the methods that retrieve endpoints.
		"""
		G = graph.DiGraph()
		G.E.add(1,2,'e1')
		G.E.add(1,3,'e2')
		G.E.add(3,4,'e3')
		G.E.add(4,5,'e4')
		
		G.N.add(6)
		
		x,y = G.endpoints('e1')
		self.assertTrue( x == 1 and y == 2 )
		
		x = G.endpoint('e2',1)
		self.assertEqual(x,3)
		
		self.assertEqual(G.target('e3'),4)
		self.assertEqual(G.source('e1'),1)

	def test_num_methods(self):
		"""
		Test the methods that keep count of the number of edges/nodes.
		"""
		G = graph.DiGraph()
		G.E.add(1,2,'e1')
		G.E.add(1,3,'e2')
		G.E.add(3,4,'e3')
		G.E.add(4,5,'e4')
		
		G.N.add(6)
		
		# The basic forms
		self.assertEqual(len(G.N),6,'Incorrect node count')
		self.assertEqual(len(G.E),4,'Incorrect edge count')
		
		# The connectivity forms
		self.assertEqual(len(G.E(u=1)),2,'Incorrect edge count')
		self.assertEqual(len(G.E(v=4)),1,'Incorrect edge count')
		self.assertEqual(len(G.E(v=4,directed=False)),2,'Incorrect edge count')
		
		self.assertEqual(len(G.E(2,2)),0,'Incorrect self-loop count')
		self.assertEqual(len(G.E(6)),0,'Incorrect self-loop count')
		self.assertEqual(len(G.E(5,4)),0,'Incorrect self-loop count')
		self.assertEqual(len(G.E(5,6)),0,'Incorrect self-loop count')
		self.assertEqual(len(G.E(4,5)),1,'Incorrect self-loop count')
		
		# remove some nodes and edges
		G.N.remove(4)
		G.E.remove('e2')
		
		self.assertEqual(len(G.E(3)),0,'Incorrect edge count')
		self.assertEqual(len(G.E(1)),1,'Incorrect edge count')
		
	def test_neighbors(self):
		
		G = graph.DiGraph()
		G.E.add(1,2)
		G.E.add(1,3)
		G.E.add(1,4)
		G.E.add(2,3)
		
		self.assertEqual(len(G.neighbors(1)),3)
		
		self.assertEqual(len(G.neighbors(2)),1)
		self.assertEqual(len(G.neighbors(2,directed=False)),2)
		
		self.assertEqual(len(G.neighbors(4)),0)
		self.assertEqual(len(G.neighbors(4,directed=False)),1)
		

####
# Code to run all the tests
if __name__ == '__main__':
	unittest.main()