"""
Unit tests for the dot file reader and writer.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import unittest

import pyweaver.graph as graph
import pyweaver.io.dot as dot
import pyweaver.constants as constants

import pyweaver.tests as tests

import os

class DOTTests(unittest.TestCase):

	def test_write(self):
		"""
		Test write capabilities of the dot library.
		"""
		filename = os.tmpnam()
		fh = open(filename,'w')
		
		VALUE = 22
		G = graph.Graph()
		G.test_field = VALUE
		
		dot.write(G,fh,constants.ALL)
		fh.close()
		
		fh = open(filename,'r')
		G = dot.read(fh)
		fh.close()
		
		self.assertTrue(hasattr(G,'test_field'))
		self.assertEqual(int(G.test_field),VALUE)
		
		return

	def test_read_all(self):
		"""
		Test the read-all capabilities of the dot library
		"""
		filename = tests.__path__[0] + '/test2.dot'
		
		# test read_all
		fh = open(filename,'r')
		graphs = dot.read_all(fh)
		fh.close()
		
		self.assertEqual(len(graphs),2)
		self.assertEqual(graphs[0].label,'Test1')
		self.assertEqual(graphs[1].label,'Test2')

		# test read_iter
		fh = open(filename,'r')
		graphs = [G for G in dot.read_iter(fh)]
		fh.close()
		
		self.assertEqual(len(graphs),2)
		self.assertEqual(graphs[0].label,'Test1')
		self.assertEqual(graphs[1].label,'Test2')
		
	def test_read_test1(self):
		"""
		Run a simple undirected test file.
		"""
		
		print 'Test File = %s' % (tests.__path__[0] + '/test1.dot')
		
		fh = open(tests.__path__[0] + '/test1.dot', 'r')
		G = dot.read(fh)
		
		self.assertFalse(G.is_directed())
		
		self.assertEqual(len(G.N),3)
		self.assertEqual(len(G.E),3)
		
		x = [n for n in G.N if n.label == 'x'][0]
		y = [n for n in G.N if n.label == 'y'][0]
		z = [n for n in G.N if n.label == 'z'][0]
		
		xy = G.E.get(x,y)
		
		self.assertEqual(x.num, 1)
		self.assertEqual(x.alpha, 'abc')
		
		self.assertEqual(z.prop1, 'A')
		
		self.assertEqual(xy.foo,'bar')
		
		self.assertTrue(G.E.contains(x,y))
		self.assertTrue(G.E.contains(x,z))
		self.assertTrue(G.E.contains(y,z))
		
		return

####
# Code to run all the tests
if __name__ == '__main__':
	unittest.main()