"""
Floyd-Warshall algorithm test functions.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import unittest
import pyweaver.graph as graph
from pyweaver.shortestpath import floyd_warshall

class FloydWarshallTests(unittest.TestCase):
	
	def test_sp_undirected(self):
		# make graph		
		G = graph.Graph()
		
		G.E.add(1,2,'e1')
		G.E.add(2,4,'e2')
		G.E.add(1,3,'e3')
		G.E.add(4,5,'e4')
		G.E.add(3,5,'e5')
		G.V.add(6)
		
		# set weights
		weights = {'e1':1., 'e2':2., 'e3':2, 'e4':0.5, 'e5':3}
		
		# find single shortest paths
		(D,P,lookup) = floyd_warshall(G,weights)
		n1 = lookup[1]
		n2 = lookup[2]
		n3 = lookup[3]
		n4 = lookup[4]
		n5 = lookup[5]
		n6 = lookup[6]
		
		self.assertEqual(D[n1][n2],1,'Wrong path length.')
		print D[n1][n5]
		self.assertEqual(D[n1][n5],3.5,'Wrong path length.')
		self.assertEqual(D[n5][n1],3.5,'Wrong path length.')
		#self.assertEqual(paths[5],solution5,'Wrong path length.')
		#self.assertEqual(paths[6],solution6,'Wrong path length.')
				
if __name__ == '__main__':
	unittest.main()