"""
Unit tests for the csv file reader and writer.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import unittest
import pyweaver.io.gcsv as gcsv

import pyweaver.tests as tests

class GCSVTests(unittest.TestCase):

	def test_read_cmatrix_test1(self):
		"""
		Run a simple connectivity matrix from a file.
		"""
		
		print 'Test File = %s' % (tests.__path__[0] + '/graph_cmatrix1.csv')
		
		fh = open(tests.__path__[0] + '/graph_cmatrix1.csv', 'r')
		G = gcsv.read_cmatrix(fh,directed=True)
		
		self.assertTrue(G.is_directed())
		
		self.assertEqual(len(G.N),4)
		self.assertEqual(len(G.E),8)
		
		a = [n for n in G.N if n.label == 'A'][0]
		b = [n for n in G.N if n.label == 'B'][0]
		c = [n for n in G.N if n.label == 'C'][0]
		d = [n for n in G.N if n.label == 'D'][0]
		
		self.assertTrue(G.E.contains(a,b))
		self.assertTrue(G.E.contains(a,c))
		self.assertFalse(G.E.contains(a,d))
		
		self.assertTrue(G.E.contains(b,a))
		self.assertFalse(G.E.contains(b,c))
		self.assertFalse(G.E.contains(b,b))		
		
		return

####
# Code to run all the tests
if __name__ == '__main__':
	unittest.main()