"""
Unit tests for the undirected graph.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import unittest
import pyweaver.generate.random_graph as gen
import pyweaver.constants as constants

class GenerateTests(unittest.TestCase):
    
    def test_random(self):
        """
        Basic sanity tests on the random generation function
        """
        
        ###
        # build a graph with a specific # of nodes and edges
        G = gen.random(nodes=100,edges=50)
        
        self.assertEqual(len(G.N),100)
        self.assertEqual(len(G.E),50)
        
        self.assertFalse(G.is_directed())
        
        ###
        # build a directed graph
        nlist = ['x','y','z','a','b','c','d','e']
        G = gen.random(graph=constants.DIRECTED, nodes=nlist, edges=3)
        
        self.assertEqual(len(G.N),len(nlist))
        self.assertEqual(len(G.E),3)
        
        self.assertTrue(G.is_directed())
        
	def test_ba(self):
		G = gen.barabasi_albert(nodes=100)
		
		self.assertEqual(len(G.V),100)
		
		return
####
# Code to run all the tests
if __name__ == '__main__':
	unittest.main()