"""
Test carlyle function in the pyweaver.algorithms.nsp module.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import unittest
import pyweaver.graph as graph
from pyweaver.shortestpath import carlyle

class CarlyleTests(unittest.TestCase):
	
	def test_carlyle_undirected(self):
		# make graph
		G = graph.Graph()
		
		G.E.add(1,2,'e1')
		G.E.add(1,4,'e2')
		G.E.add(1,3,'e3')
		G.E.add(4,3,'e4')
		G.E.add(3,5,'e5')
		G.E.add(4,5,'e6')
		G.E.add(2,5,'e7')
		
		# set weights
		weights = {'e1':0.5, 'e2':1., 'e3':1.5, 'e4':0.4, 'e5':0.5, 'e6':2, 'e7':0.5}
		
		# find near-shortest simple paths
		paths = carlyle(G,weights,1,5,1.)
		
		p1 = ['e1','e7']
		p2 = ['e2','e4','e5']
		p3 = ['e3','e5']
		
		solution = [p1, p2, p3]
		
		self.assertEqual(paths,solution)
			
if __name__ == '__main__':
	unittest.main()