"""
Unit tests for the tree parsimony implementation.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import unittest
from pyweaver.parsimony import *
from pyweaver.tree import *

class ParsimonyTests(unittest.TestCase):
	
	def test_basic(self):
		"""
		Basic sanity tests on the parsimony function
		"""
		
		# make a tree
		root = Node()
		T = RTree(root)
		
		l1 = Node('l1')
		l2 = Node('l2')
		l3 = Node('l3')
		l4 = Node('l4')
		l5 = Node('l5')
		
		i1 = Node()
		i2 = Node()
		i3 = Node()
		
		T.add_child(root,i1)
		T.add_child(root,i2)
		T.add_child(i1,l1)
		T.add_child(i1,l2)
		T.add_child(i2,l3)
		T.add_child(i2,i3)
		T.add_child(i3,l4)
		T.add_child(i3,l5)

		vectors = { l1 : ['a'], l2 : ['b'], l3 : ['a'], l4 : ['b'], l5 : ['b']}

		# compute parsimony
		pscore = compute_parsimony_score(T,vectors)
		self.assertEquals(pscore,2)

		######
		# Try another tree
		T = RTree(root)
		
		l1 = Node('l1')
		l2 = Node('l2')
		l3 = Node('l3')
		l4 = Node('l4')
		l5 = Node()
		l6 = Node()
		l7 = Node()
		l8 = Node()
		l9 = Node()
		
		i1 = Node()
		i2 = Node()
		i3 = Node()
		i4 = Node()
		i5 = Node()
		i6 = Node()
		i7 = Node()
		
		T.add_child(root,i1)
		T.add_child(root,i3)
		T.add_child(i1,i2)
		T.add_child(i1,l3)
		T.add_child(i2,l1)
		T.add_child(i2,l2)
		T.add_child(i3,i7)
		T.add_child(i3,i4)
		T.add_child(i4,l6)
		T.add_child(i4,i5)
		T.add_child(i5,l7)
		T.add_child(i5,i6)
		T.add_child(i6,l8)
		T.add_child(i6,l9)
		T.add_child(i7,l4)
		T.add_child(i7,l5)

		vectors = { l1 : [0], l2 : [1], 
					l3 : [2], l4 : [2], 
					l5 : [1], l6 : [3],
					l7 : [0], l8 : [0],
					l9 : [1]}
					
		# compute parsimony
		pscore = compute_parsimony_score(T,vectors)
		self.assertEquals(pscore,6)
		
####
# Code to run all the tests
if __name__ == '__main__':
	unittest.main()