"""
Path utility functions unit tests.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import unittest
import pyweaver.graph as graph
import pyweaver.path as pu

class PathTests(unittest.TestCase):
	
	def test_weight(self):
		# make graph		
		G = graph.Graph()
		
		G.E.add(1,2,'e1')
		G.E.add(2,4,'e2')
		G.E.add(1,3,'e3')
		G.E.add(4,5,'e4')
		G.E.add(3,5,'e5')
		G.N.add(6)
		
		# set weights
		weights = {'e1':1., 'e2':2., 'e3':2, 'e4':0.5, 'e5':3}
		
		path1 = ['e1','e2','e4']
		
		self.assertEqual(pu.get_weight(path1,weights),3.5)
	
	def test_path2node_list_udir(self):
		# make an undirected graph		
		G = graph.Graph()
		
		G.E.add(1,2,'e1')
		G.E.add(2,4,'e2')
		G.E.add(1,3,'e3')
		G.E.add(4,5,'e4')
		G.E.add(3,5,'e5')
		G.N.add(6)
		
		# make a path
		path1 = ['e1','e2','e4']
		
		nlist = pu.get_node_list(G,path1)
		
	def test_path2node_list_udir(self):
		# make an undirected graph		
		G = graph.DiGraph()
		
		G.E.add(1,2,'e1')
		G.E.add(2,4,'e2')
		G.E.add(1,3,'e3')
		G.E.add(4,5,'e4')
		G.E.add(3,5,'e5')
		G.N.add(6)
		
		# make a path
		path1 = ['e1','e2','e4']
		
		nlist = pu.get_node_list(G,path1)
		
		self.assertEqual(nlist,[1,2,4,5])
		
	def test_sorting(self):
		# make graph		
		G = graph.Graph()
		
		G.E.add(1,2,'e1')
		G.E.add(2,4,'e2')
		G.E.add(1,3,'e3')
		G.E.add(4,5,'e4')
		G.E.add(3,5,'e5')
		G.N.add(6)
		
		# set weights
		weights = {'e1':1., 'e2':2., 'e3':2, 'e4':0.5, 'e5':3}
		
		path1 = ['e1','e2','e4']
		path2 = ['e1','e2']
		path3 = ['e3','e5']
		
		paths = [path1,path2,path3]
		
		paths.sort(pu.build_weight_comparator(weights))
		
		self.assertEqual(paths,[path2,path1,path3])
		
####
# Code to run all the tests
if __name__ == '__main__':
	unittest.main()