"""
Unit tests for the tree data structures.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import unittest
from pyweaver.tree import *

class RootedTreeGraphTests(unittest.TestCase):

	def test_newick(self):
		"""
		Test the method that returns a newick representation of a tree.
		"""
		T = RTree('R')
		T.add_child('R','C1')
		T.add_child('R','C2')
		T.add_child('C1','X1')
		T.add_child('C1','X2')
		
		self.assertTrue(T.is_rooted())
		
		self.assertEqual(T.parent('C1'),'R')
		self.assertEqual(T.parent('X2'),'C1')
		
		nstr = T.newick()
		self.assertEqual(nstr,'((X1, X2)C1, C2)R;')
		
####
# Code to run all the tests
if __name__ == '__main__':
	unittest.main()