"""
This module provides implementations of various measures of distances.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import math

__all__ = ['rmsd']

def rmsd(v1,v2,length = None):
	"""
	Compute the root-mean squared difference between vectors v1 and v2.  The vectors must be of the same length
	and be iterable.  If v1 and v2 are iterable, but do not support the len(...) function, then the length of
	these vectors must be specified via the length argument.
	"""
	
	if length is None:
		length = len(v1)
		
	length = float(length)
	d = math.sqrt(float(sum([pow(x - y,2.) for x,y in zip(v1,v2)])) / length)
	
	return d

#####
# For testing...
def main():
	pass

if __name__ == '__main__':
	main()

