"""
This module provides an complete implementation of a min heap.  Mainly this fills out
the functionality that are not included in the heapq module provided in the standard
distribution of Python.

A heap is a list.  In order to guarantee correct operation of the heap, only heap
functions should be used to manipulate the list.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

import heapq

__all__ = ['heapify','push','pop','decrease_key']

def heapify(heap):
    """
    Take all the objects in the current list and organize them
    into a heap.
    """
    heapq.heapify(heap)

def push(heap,item):
    """
    Push a new value into the heap.
    """
    heapq.heappush(item,heap)

def pop(heap):
    """
    Remove the min value from the heap and adjust the heap.
    """
    return heapq.heappop(heap)

def decrease_key(heap,item):
    """
    Adjust the key of an object downward.  Since heap-able objects are comparable
    themselves, this method is actually a notification that the key of the specified
    object has changed and the heap must be updated.
    """
    # get the position of the object
    i = 0
    while i < len(heap):
        if id(item) == id(heap[i]):
            break
        i += 1

    if i == len(heap):
        raise Exception, 'Item %s does not exist in the heap' % str(item)
    
    while i > 0:
        # get the parent
        pidx = (i - 1) >> 1
        pitem = heap[pidx]

        # check the heap condition
        if pitem <= item:
            break
        
        heap[i] = heap[pidx]
        i = pidx

    heap[i] = item

    return
