"""
This module provides various functions that are useful for working with sets.
"""

"""
Copyright 2007 Derek Ruths

This file is part of PyWeaver.

PyWeaver is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyWeaver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with PyWeaver.  If not, see <http://www.gnu.org/licenses/>.
"""

__all__ = ['subset_iter']

def subset_iter(S,k):
    """
    Iterate over all k-size subsets of set S.  The set object returned by the iterator should
    not be modified as it is recycled for use during each subsequent iteration.
    """
    S_list = list(S)
    X = set()
    
    active_bit = k-1
    bit_pos = range(0,k)
    
    # this is a slightly weird initialization point, but it's expedient for getting the
    # furthest right bit to start out in the correct spot.
    bit_pos[active_bit] -= 1
    
    while active_bit >= 0:
        
        if bit_pos[active_bit] == (len(S_list) - 1) or (active_bit < (k-1) and bit_pos[active_bit+1] == (bit_pos[active_bit] + 1)):
            active_bit -= 1
            continue
        
        # update the active bit
        bit_pos[active_bit] += 1
        
        # reset all the bit ahead of this bit
        for bit in range(active_bit+1,k):
            bit_pos[bit] = bit_pos[bit-1] + 1
            
        # start at the LSB again
        active_bit = k-1
        
        # generate the next subset
        X.clear()
        for pos in bit_pos:
            X.add(S_list[pos])

        yield X
        
    return
